/*
 * encoder.c
 *
 *  Created on: 31.12.2014
 *      Author: daniel
 */


#include <stdbool.h>
#include <stdint.h>
#include "stm32f4xx_conf.h"

static struct {
	uint32_t maxValue;
	uint32_t minValue;
	uint32_t value;
	bool changed;
	uint32_t offset;
} encoderValues;

void encoderInit(int32_t maxValue, int32_t minValue, int32_t startValue) {
	GPIO_InitTypeDef GPIO_InitStructure;

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM5, ENABLE);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource0, GPIO_AF_TIM5);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_TIM5);

	encoderValues.changed=false;
	encoderValues.offset=10000;
	encoderValues.maxValue=maxValue+encoderValues.offset;
	encoderValues.minValue=minValue+encoderValues.offset;
	encoderValues.value=startValue+encoderValues.offset;

	TIM_EncoderInterfaceConfig(TIM5, TIM_EncoderMode_TI12, TIM_ICPolarity_Rising, TIM_ICPolarity_Rising);

	TIM_Cmd(TIM5, ENABLE);
	TIM_SetCounter(TIM5,encoderValues.value);
}

bool encoderGetChanged() {
	volatile uint32_t counter = TIM_GetCounter(TIM5);
	if(counter!=encoderValues.value) {
		encoderValues.changed=true;
		if(counter<encoderValues.minValue) {
			counter=encoderValues.minValue;
			TIM_SetCounter(TIM5,counter);
		}
		if(counter>encoderValues.maxValue) {
			counter=encoderValues.maxValue;
			TIM_SetCounter(TIM5,counter);
		}
		encoderValues.value=counter;
	}
	return(encoderValues.changed);
}

int32_t encoderGetValue() {
	int32_t returnval;
	encoderValues.changed=false;
	returnval=encoderValues.value;
	returnval-=encoderValues.offset;
	return(returnval);
}


void encoderSetMax(uint32_t newMax) {
	encoderValues.maxValue=newMax+encoderValues.offset;
	if(encoderValues.value>encoderValues.maxValue) {
		encoderValues.value=encoderValues.maxValue;
		TIM_SetCounter(TIM5,encoderValues.value);
		encoderValues.changed=true;
	}
}

void encoderSetMin(uint32_t newMin) {
	encoderValues.minValue=newMin+encoderValues.offset;
	if(encoderValues.value<encoderValues.minValue) {
		encoderValues.value=encoderValues.minValue;
		TIM_SetCounter(TIM5,encoderValues.value);
		encoderValues.changed=true;
	}
}
