/*
 * io_pins.c
 *
 *  Created on: 04.01.2015
 *      Author: daniel
 */

#include "mp3util.h"
#include "stm32f4xx_conf.h"
#include "global_status.h"
#include "staticData.h"
#include "audio.h"

typedef union  {
	struct {
		uint8_t dir0 :1;
		uint8_t dir1 :1;
		uint8_t dir2 :1;
		uint8_t dir3 :1;
		uint8_t dir4 :1;
		uint8_t dir5 :1;
		uint8_t dir6 :1;
		uint8_t dir7 :1;
		uint8_t dir8 :1;
		uint8_t dir9 :1;
		uint8_t play :1;
		uint8_t skip :1;
		uint8_t rewind :1;
		uint8_t encoder :1;
		uint8_t power :1;
	} button;
	uint16_t raw;
} GButtons;
GButtons gButtons;

void buttonsAndOutputInit() {
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

	GPIO_InitTypeDef  GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_8| GPIO_Pin_9| GPIO_Pin_10 |
			GPIO_Pin_11 | GPIO_Pin_12| GPIO_Pin_13| GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init(GPIOE, &GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2| GPIO_Pin_3;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// Transistor for power off relais
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
	GPIO_Init(GPIOD, &GPIO_InitStructure);

	// Configure PD12, PD13, PD14 and PD15 in output pushpull mode
	// onboard LEDs
	//GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_13| GPIO_Pin_14| GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13| GPIO_Pin_14| GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO_InitStructure);

	// power button to 5V with pulldown
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
	GPIO_Init(GPIOC, &GPIO_InitStructure);


}

void powerOff() {
	// power the reset coul of bistable relais. it will then power off the device.
	// we release the signal anyway, if no relais is connected or if supply is bridged during development.
	GPIO_SetBits(GPIOD, GPIO_Pin_8);
//	Delay(100);  // need to do it without irq for usb disconnect handler
	for (volatile uint32_t i=0; i<10000000; i++) {
		GPIO_SetBits(GPIOD, GPIO_Pin_8);
	}
	GPIO_ResetBits(GPIOD, GPIO_Pin_8);
}


bool getButtons() {
	gButtons.raw=0;
	gButtons.button.dir9 = (GPIO_ReadInputDataBit(GPIOE, GPIO_Pin_7) == 0);
	gButtons.button.dir8 = (GPIO_ReadInputDataBit(GPIOE, GPIO_Pin_8) == 0);
	gButtons.button.dir7 = (GPIO_ReadInputDataBit(GPIOE, GPIO_Pin_9) == 0);
	gButtons.button.dir6 = (GPIO_ReadInputDataBit(GPIOE, GPIO_Pin_10) == 0);
	gButtons.button.dir5 = (GPIO_ReadInputDataBit(GPIOE, GPIO_Pin_11) == 0);
	gButtons.button.dir4 = (GPIO_ReadInputDataBit(GPIOE, GPIO_Pin_12) == 0);
	gButtons.button.dir3 = (GPIO_ReadInputDataBit(GPIOE, GPIO_Pin_13) == 0);
	gButtons.button.dir2 = (GPIO_ReadInputDataBit(GPIOE, GPIO_Pin_14) == 0);
	gButtons.button.dir1 = (GPIO_ReadInputDataBit(GPIOE, GPIO_Pin_15) == 0);
	gButtons.button.dir0 = (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_13) == 0);
	gButtons.button.play = (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_2) == 0);
	gButtons.button.skip = (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_3) == 0);
	gButtons.button.rewind = (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_14) == 0);
	gButtons.button.encoder = (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_15) == 0);
	gButtons.button.power = (GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_1) == 1);
	return (gButtons.raw!=0);

}


// returns true on folder button press, false otherwise
bool buttonsHandler() {
	static uint32_t debounce=0;

	if(debounce>0) {
		if(getButtons()) {
			// wait for button release
		} else {
			debounce--;
		}
	} else {
		getButtons();
		if(gButtons.raw>0) {
			for(uint8_t i=0; i<10; i++) {
				if((gButtons.raw>>i)&0x01) {
					golbalStatusPlayDir(i);
					debounce=20;
					return (true);
					break;
				}
			}
			if(gButtons.button.play) {
				globalStatusTogglePause();
				debounce=20;
			}
			if(gButtons.button.skip) {
				globalStatusSkip();
				debounce=20;
			}
			if(gButtons.button.rewind) {
				globalStatusRewind();
				debounce=20;
			}
			if(gButtons.button.power) {
				globalStatusPowerOff();
				debounce=20;
			}
			if(gButtons.button.encoder) {
				globalStatusToggleTone();
				debounce=20;
			}
		}
	}
	return (false);
}

