/*
 * mp3util.c
 *
 *  Created on: 23.12.2014
 *      Author: daniel
 */

#include "mp3util.h"
#include "mp3dec.h"
#include "Audio.h"
#include "global_status.h"
#include <string.h>


HMP3Decoder				hMP3Decoder;
// Variables
volatile uint32_t		time_var1, time_var2;

const char *get_filename_ext(const char *filename) {
	const char *dot = strrchr(filename, '.');
	if(!dot || dot == filename) return "";
	return dot + 1;
}



MP3FrameInfo getMp3Info(FIL *f) {
	unsigned char buffer[4096];
	UINT read;
	int result;
	uint32_t filepos=f_tell(f);
	MP3FrameInfo info;
	info.bitrate=0;
	info.samprate=0;
	info.bitsPerSample=0;
	do{
		if(f_read(f, buffer, sizeof(buffer), &read)!=FR_OK) {
			return info;
		}
		result=MP3FindSyncWord(buffer, sizeof(buffer));
		if(result<0) {
			filepos+=sizeof(buffer);
		}
	} while(result<0);
	filepos+=result;
	f_lseek(f, filepos);
	if(f_read(f, buffer, sizeof(buffer), &read)!=FR_OK) {
		return info;
	}

	hMP3Decoder = MP3InitDecoder();
	MP3GetNextFrameInfo(hMP3Decoder, &info,buffer);

	f_lseek(f, 0);

	return (info);

}

/*
 * Taken from
 * http://www.mikrocontroller.net/topic/252319
 */
static uint32_t Mp3ReadId3V2Text(FIL* pInFile, uint32_t unDataLen, char* pszBuffer, uint32_t unBufferSize)
{
	UINT unRead = 0;
	BYTE byEncoding = 0;
	if((f_read(pInFile, &byEncoding, 1, &unRead) == FR_OK) && (unRead == 1))
	{
		unDataLen--;
		if(unDataLen <= (unBufferSize - 1))
		{
			if((f_read(pInFile, pszBuffer, unDataLen, &unRead) == FR_OK) ||
					(unRead == unDataLen))
			{
				if(byEncoding == 0)
				{
					// ISO-8859-1 multibyte
					// just add a terminating zero
					pszBuffer[unDataLen] = 0;
				}
				else if(byEncoding == 1)
				{
					// UTF16LE unicode
					uint32_t r = 0;
					uint32_t w = 0;
					if((unDataLen > 2) && (pszBuffer[0] == 0xFF) && (pszBuffer[1] == 0xFE))
					{
						// ignore BOM, assume LE
						r = 2;
					}
					for(; r < unDataLen; r += 2, w += 1)
					{
						// should be acceptable for 7 bit ascii
						pszBuffer[w] = pszBuffer[r];
					}
					pszBuffer[w] = 0;
				}
			}
			else
			{
				return 1;
			}
		}
		else
		{
			// we won't read a partial text
			if(f_lseek(pInFile, f_tell(pInFile) + unDataLen) != FR_OK)
			{
				return 1;
			}
		}
	}
	else
	{
		return 1;
	}
	return 0;
}


/*
 * Taken from
 * http://www.mikrocontroller.net/topic/252319
 */
uint32_t Mp3ReadId3V2Tag(FIL* pInFile, char* pszArtist, uint32_t unArtistSize, char* pszTitle, uint32_t unTitleSize)
{
	pszArtist[0] = 0;
	pszTitle[0] = 0;

	BYTE id3hd[10];
	UINT unRead = 0;
	if((f_read(pInFile, id3hd, 10, &unRead) != FR_OK) || (unRead != 10))
	{
		return 1;
	}
	else
	{
		uint32_t unSkip = 0;
		if((unRead == 10) &&
				(id3hd[0] == 'I') &&
				(id3hd[1] == 'D') &&
				(id3hd[2] == '3'))
		{
			unSkip += 10;
			unSkip = ((id3hd[6] & 0x7f) << 21) | ((id3hd[7] & 0x7f) << 14) | ((id3hd[8] & 0x7f) << 7) | (id3hd[9] & 0x7f);

			// try to get some information from the tag
			// skip the extended header, if present
			uint8_t unVersion = id3hd[3];
			if(id3hd[5] & 0x40)
			{
				BYTE exhd[4];
				f_read(pInFile, exhd, 4, &unRead);
				size_t unExHdrSkip = ((exhd[0] & 0x7f) << 21) | ((exhd[1] & 0x7f) << 14) | ((exhd[2] & 0x7f) << 7) | (exhd[3] & 0x7f);
				unExHdrSkip -= 4;
				if(f_lseek(pInFile, f_tell(pInFile) + unExHdrSkip) != FR_OK)
				{
					return 1;
				}
			}
			uint32_t nFramesToRead = 2;
			while(nFramesToRead > 0)
			{
				char frhd[10];
				if((f_read(pInFile, frhd, 10, &unRead) != FR_OK) || (unRead != 10))
				{
					return 1;
				}
				if((frhd[0] == 0) || (strncmp(frhd, "3DI", 3) == 0))
				{
					break;
				}
				char szFrameId[5] = {0, 0, 0, 0, 0};
				memcpy(szFrameId, frhd, 4);
				uint32_t unFrameSize = 0;
				uint32_t i = 0;
				for(; i < 4; i++)
				{
					if(unVersion == 3)
					{
						// ID3v2.3
						unFrameSize <<= 8;
						unFrameSize += frhd[i + 4];
					}
					if(unVersion == 4)
					{
						// ID3v2.4
						unFrameSize <<= 7;
						unFrameSize += frhd[i + 4] & 0x7F;
					}
				}

				if(strcmp(szFrameId, "TPE1") == 0)
				{
					// artist
					if(Mp3ReadId3V2Text(pInFile, unFrameSize, pszArtist, unArtistSize) != 0)
					{
						break;
					}
					nFramesToRead--;
				}
				else if(strcmp(szFrameId, "TIT2") == 0)
				{
					// title
					if(Mp3ReadId3V2Text(pInFile, unFrameSize, pszTitle, unTitleSize) != 0)
					{
						break;
					}
					nFramesToRead--;
				}
				else
				{
					if(f_lseek(pInFile, f_tell(pInFile) + unFrameSize) != FR_OK)
					{
						return 1;
					}
				}
			}
		}
		if(f_lseek(pInFile, unSkip) != FR_OK)
		{
			return 1;
		}
	}

	return 0;
}

bool findNextFolder(const char *basefolder, char *lastFolderName, char *newFolderName) {
	DIR dir;
	FILINFO fno;
	bool returnval=false;
	bool first=true;
	char current[256];
	strcpy(current, lastFolderName);
#if _USE_LFN
	static char lfn[_MAX_LFN + 1];
	fno.lfname = lfn;
	fno.lfsize = sizeof(lfn);
#endif

	FRESULT res;
	char *fn;
	res=f_opendir(&dir, basefolder);
	if (res == FR_OK) {
		for (;;) {
			res = f_readdir(&dir, &fno);                   // Read a directory item
			if (res != FR_OK || fno.fname[0] == 0) {
				break;  // Break on error or end of dir
			}
			if (fno.fname[0] == '.') {
				continue;             // Ignore dot entry
			}
			printf("S: %s\r\n",fno.fname);
#if _USE_LFN
			fn = *fno.lfname ? fno.lfname : fno.fname;
			printf("L: %s\r\n",fn);
#else
			fn = fno.fname;
#endif
			if (fno.fattrib & AM_DIR) {                    // It is a directory
				if(first) {
					if((strcoll(current,fn)<0) || (current[0]=='\0')) {
						strcpy(newFolderName,fno.fname);
						strcpy(current,fn);
						returnval=true;
						first=false;
					}
				} else {
					if ((strcoll(current,fn)>0) && ((strcoll(lastFolderName,fn)<0)|| (lastFolderName[0]=='\0'))) {
						strcpy(newFolderName,fno.fname);
						strcpy(current,fn);
						returnval=true;
					}
				}
				if (res != FR_OK) {
					break;
				}
			} else {                                       // It is a file.
				//				printf("%s/%s\n", path, fn);
			}
		}

		f_closedir(&dir);
	}
	if(returnval) {
		strcpy(lastFolderName, current);
	}
	return (returnval);
}


bool findNextMp3(const char *basefolder, char *longFileName, char *shortFileName) {
	DIR dir;
	FILINFO fno;
	bool returnval=false;
	bool first=true;
	char current[256];
	strcpy(current, longFileName);
#if _USE_LFN
	static char lfn[_MAX_LFN + 1];
	fno.lfname = lfn;
	fno.lfsize = sizeof(lfn);
#endif

	FRESULT res;
	char *fn;
	res=f_opendir(&dir, basefolder);
	if (res == FR_OK) {
		for (;;) {
			res = f_readdir(&dir, &fno);                   // Read a directory item
			if (res != FR_OK || fno.fname[0] == 0) {
				break;  // Break on error or end of dir
			}
			if (fno.fname[0] == '.') {
				continue;             // Ignore dot entry
			}
			printf("S: %s\r\n",fno.fname);
#if _USE_LFN
			fn = *fno.lfname ? fno.lfname : fno.fname;
			printf("L: %s\r\n",fn);
#else
			fn = fno.fname;
#endif
			if (fno.fattrib & AM_DIR) {                    // It is a directory; we cannot do subdirs
				continue;
			} else {                                       // It is a file.
				if (strcmp("mp3", get_filename_ext(fn)) == 0) {
					if(first) {
						if((strcoll(current,fn)<0) || (current[0]=='\0')) {
							strcpy(shortFileName,fno.fname);
							strcpy(current,fn);
							returnval=true;
							first=false;
						}
					} else {
						if ((strcoll(current,fn)>0) && ((strcoll(longFileName,fn)<0)|| (longFileName[0]=='\0'))) {
							strcpy(shortFileName,fno.fname);
							strcpy(current,fn);
							returnval=true;
						}
					}
					if (res != FR_OK) {
						break;
					}
				} else {
					continue;
				}
			}
		}

		f_closedir(&dir);
	}
	if(returnval) {
		strcpy(longFileName, current);
	}
	return (returnval);
}

/*
 * Called by the SysTick interrupt
 */
void TimingDelay_Decrement(void) {
	if (time_var1) {
		time_var1--;
	}
	time_var2++;
	globalStatusSystick();
}

/*
 * Delay a number of systick cycles (1ms)
 */
void Delay(volatile uint32_t nTime) {
	time_var1 = nTime;
	while(time_var1){};
}

