/*
 * staticData.c
 *
 *  Created on: 24.01.2015
 *      Author: daniel
 */

#include <string.h>
#include <stdint.h>

#include "stm32f4xx.h"
#include "stm32f4xx_it.h"
#include "stm32f4xx_conf.h"
#include "staticData.h"

/* initcode is used to detect if battery backed ram is initialized.
 * if the structure is changed, the initcode needs to be changed also, to force new initialization of the memory on next startup
 */
#define INITCODE 0xabcd1236

typedef struct {
	struct {
		char shortname[12];
		uint32_t currentFile;
		uint32_t positionInFile;
	} folder[10]; // needs 20 Bytes each
	uint32_t initialized;  // set to INITCODE for initialisation
	uint8_t volume;
	int8_t toneH;
	int8_t toneL;
	uint32_t currentFolderNumber;
} StaticData;
//#define staticData         (*((volatile StaticData *) 0x40024000))

StaticData __attribute__((section (".myStaticSection"))) staticData;

void initStaticSram() {
	// battery backed SRAM
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
	PWR_BackupAccessCmd(ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_BKPSRAM, ENABLE);
	PWR_BackupRegulatorCmd(ENABLE);
	while (PWR_GetFlagStatus(PWR_FLAG_BRR) == RESET) {
	}
	if (staticData.initialized != INITCODE) {
		staticData.initialized = INITCODE;
		staticData.volume=INITIALVOLUME;
		staticData.toneH=0;
		staticData.toneL=0;
		for (uint8_t i = 0; i < 10; i++) {
			staticData.folder[i].shortname[0] = '\0';
			staticData.folder[i].currentFile = 0;
			staticData.folder[i].positionInFile = 0;
		}
	}
}

void staticDataSetPlaybackFile(const uint8_t foldernumber, const uint32_t currentFile) {
	staticData.currentFolderNumber=foldernumber;
	staticData.folder[foldernumber].currentFile=currentFile;
}

void staticDataResetPlaybackPosition() {
	staticData.folder[staticData.currentFolderNumber].positionInFile=0;

}
/*
void staticDataIncPlaybackPosition(const uint32_t bytesRead) {
	staticData.folder[staticData.currentFolderNumber].positionInFile+=bytesRead;
}
*/
void staticDataSetPlaybackPosition(const uint32_t bytesRead) {
	staticData.folder[staticData.currentFolderNumber].positionInFile=bytesRead;
}

uint32_t staticDataGetCurrentFile(const uint8_t foldernumber) {
	return(staticData.folder[foldernumber].currentFile);
}
uint32_t staticDataGetpositionInFile() {
	return(staticData.folder[staticData.currentFolderNumber].positionInFile);
}

uint8_t staticDataGetVolume() {
	return ( staticData.volume);
}
void staticDataSetVolume(uint8_t newVolume) {
	staticData.volume=newVolume;
}

int8_t staticDataGetToneL() {
	return ( staticData.toneL);
}
void staticDataSetToneL(int8_t newVolume) {
	staticData.toneL=newVolume;
}

int8_t staticDataGetToneH() {
	return ( staticData.toneH);
}
void staticDataSetToneH(int8_t newVolume) {
	staticData.toneH=newVolume;
}

int staticDataCompareFoldername(const uint8_t foldernumber, const char *shortname) {
	return (strcoll(shortname, staticData.folder[foldernumber].shortname));
}

void staticDataSetFoldername(const uint8_t foldernumber, const char *shortname) {
	strcpy(staticData.folder[foldernumber].shortname, shortname);
	staticData.folder[foldernumber].currentFile=0;
	staticData.folder[foldernumber].positionInFile=0;
}

void staticDataRewind(const uint8_t directory) {
	if(directory < 10) {
		staticData.currentFolderNumber=directory;
		if(staticData.folder[staticData.currentFolderNumber].currentFile >= 1) {
			staticData.folder[staticData.currentFolderNumber].currentFile-=1;
		} else {
			staticData.folder[staticData.currentFolderNumber].currentFile=0;
		}
		staticData.folder[staticData.currentFolderNumber].positionInFile=0;
	}
}



